@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.c,v 1.11 2006/01/06 16:50:49 pborelli Exp $
+ * $Id: gedit-window.c,v 1.20 2006/03/03 16:31:17 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -185,7 +185,7 @@ gedit_window_window_state_event (GtkWidget           *widget,
 		gboolean show;
 
 		show = !(event->new_window_state &
-		         (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN));
+			(GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN));
 
 		_gedit_statusbar_set_has_resize_grip (GEDIT_STATUSBAR (window->priv->statusbar),
 						      show);
@@ -324,7 +324,7 @@ gedit_window_class_init (GeditWindowClass *klass)
 
 static void
 menu_item_select_cb (GtkMenuItem *proxy,
-                     GeditWindow *window)
+		     GeditWindow *window)
 {
 	GtkAction *action;
 	char *message;
@@ -560,6 +560,11 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 					      "SearchFindPrevious");
 	gtk_action_set_sensitive (action, state_normal && b);
 
+	action = gtk_action_group_get_action (window->priv->action_group,
+					      "SearchClearHighlight");
+	gtk_action_set_sensitive (action, state_normal && b);
+
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "SearchGoToLine");
 	gtk_action_set_sensitive (action, state_normal);
@@ -850,7 +855,7 @@ recent_tooltip_func_gtk (GtkTooltips   *tooltips,
 
 	trunc_uri = gedit_utils_str_middle_truncate (uri_for_display,
 						     TIP_MAX_URI_LEN);
-        g_free (uri_for_display);
+	g_free (uri_for_display);
 
 	/* Translators: %s is a URI */
 	tip = g_strdup_printf (_("Open '%s'"), trunc_uri);
@@ -875,7 +880,7 @@ recent_tooltip_func_uim (EggRecentItem *item,
 
 	trunc_uri = gedit_utils_str_middle_truncate (uri_for_display,
 						     TIP_MAX_URI_LEN);
-        g_free (uri_for_display);
+	g_free (uri_for_display);
 
 	/* Translators: %s is a URI */
 	tip = g_strdup_printf (_("Open '%s'"), trunc_uri);
@@ -928,7 +933,26 @@ set_non_homogeneus (GtkWidget *widget, gpointer data)
 {
 	gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (widget), FALSE);
 }
-                                             
+
+static void
+toolbar_visibility_changed (GtkWidget   *toolbar,
+			    GeditWindow *window)
+{
+	gboolean visible;
+	GtkAction *action;
+
+	visible = GTK_WIDGET_VISIBLE (toolbar);
+
+	if (gedit_prefs_manager_toolbar_visible_can_set ())
+		gedit_prefs_manager_set_toolbar_visible (visible);
+
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
+					      "ViewToolbar");
+
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
+}
+
 static void
 create_menu_bar_and_toolbar (GeditWindow *window, 
 			     GtkWidget   *main_box)
@@ -971,6 +995,10 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 				      gedit_menu_entries,
 				      G_N_ELEMENTS (gedit_menu_entries),
 				      window);
+	gtk_action_group_add_toggle_actions (action_group,
+					     gedit_toggle_menu_entries,
+					     G_N_ELEMENTS (gedit_toggle_menu_entries),
+					     window);
 
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	g_object_unref (action_group);
@@ -1002,10 +1030,6 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 	window->priv->quit_action_group = action_group;
 
 	/* now load the UI definition */
-	if (gtk_ui_manager_add_ui_from_file (manager, "gedit-ui.xml", NULL)) // REMOVE ME... just to allow running without install
-		;
-	else
-
 	gtk_ui_manager_add_ui_from_file (manager, GEDIT_UI_DIR "gedit-ui.xml", &error);
 	if (error != NULL)
 	{
@@ -1104,7 +1128,16 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 
 	gtk_container_foreach (GTK_CONTAINER (window->priv->toolbar),
 			       (GtkCallback)set_non_homogeneus,
-			       NULL);	       
+			       NULL);
+
+	g_signal_connect_after (G_OBJECT (window->priv->toolbar),
+				"show",
+				G_CALLBACK (toolbar_visibility_changed),
+				window);
+	g_signal_connect_after (G_OBJECT (window->priv->toolbar),
+				"hide",
+				G_CALLBACK (toolbar_visibility_changed),
+				window);
 }
 
 static void
@@ -1250,6 +1283,25 @@ set_statusbar_style (GeditWindow *window,
 	return visible;
 }
 
+static void
+statusbar_visibility_changed (GtkWidget   *statusbar,
+			      GeditWindow *window)
+{
+	gboolean visible;
+	GtkAction *action;
+
+	visible = GTK_WIDGET_VISIBLE (statusbar);
+
+	if (gedit_prefs_manager_statusbar_visible_can_set ())
+		gedit_prefs_manager_set_statusbar_visible (visible);
+
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
+					      "ViewStatusbar");
+
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
+}
+
 static void
 create_statusbar (GeditWindow *window, 
 		  GtkWidget   *main_box)
@@ -1267,23 +1319,34 @@ create_statusbar (GeditWindow *window,
 			  window->priv->statusbar,
 			  FALSE, 
 			  TRUE, 
-			  0);	
+			  0);
 
-	set_statusbar_style (window, NULL);		
+	g_signal_connect_after (G_OBJECT (window->priv->statusbar),
+				"show",
+				G_CALLBACK (statusbar_visibility_changed),
+				window);
+	g_signal_connect_after (G_OBJECT (window->priv->statusbar),
+				"hide",
+				G_CALLBACK (statusbar_visibility_changed),
+				window);
+
+	set_statusbar_style (window, NULL);
 }
 
 static GeditWindow *
 clone_window (GeditWindow *origin)
 {
 	GtkWindow *window;
+	GdkScreen *screen;
 	GeditApp  *app;
 
 	gedit_debug (DEBUG_WINDOW);	
 
 	app = gedit_app_get_default ();
-	
-	window = GTK_WINDOW (gedit_app_create_window (app));
-	
+
+	screen = gtk_window_get_screen (GTK_WINDOW (origin));
+	window = GTK_WINDOW (gedit_app_create_window (app, screen));
+
 	gtk_window_set_default_size (window, 
 				     origin->priv->width,
 				     origin->priv->height);
@@ -1917,6 +1980,10 @@ can_search_again (GeditDocument *doc,
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "SearchFindPrevious");
 	gtk_action_set_sensitive (action, sensitive);
+
+	action = gtk_action_group_get_action (window->priv->action_group,
+					      "SearchClearHighlight");
+	gtk_action_set_sensitive (action, sensitive);
 }
 
 static void
@@ -2403,6 +2470,57 @@ notebook_popup_menu (GtkNotebook *notebook,
 	return FALSE;
 }
 
+static void
+hpaned_restore_position (GtkWidget   *widget,
+			 GeditWindow *window)
+{
+	gint pos;
+
+	pos = MAX (100, gedit_prefs_manager_get_side_panel_size ());
+	gtk_paned_set_position (GTK_PANED (window->priv->hpaned), pos);
+
+	/* run this only once */
+	g_signal_handlers_disconnect_by_func (widget, hpaned_restore_position, window);
+}
+
+static void
+vpaned_restore_position (GtkWidget   *widget,
+			 GeditWindow *window)
+{
+	gint pos;
+
+	pos = widget->allocation.height -
+	      MAX (50, gedit_prefs_manager_get_bottom_panel_size ());
+	gtk_paned_set_position (GTK_PANED (window->priv->vpaned), pos);
+
+	/* run this only once */
+	g_signal_handlers_disconnect_by_func (widget, vpaned_restore_position, window);
+}
+
+static void
+side_panel_visibility_changed (GtkWidget   *side_panel,
+			       GeditWindow *window)
+{
+	gboolean visible;
+	GtkAction *action;
+
+	visible = GTK_WIDGET_VISIBLE (side_panel);
+
+	if (gedit_prefs_manager_side_pane_visible_can_set ())
+		gedit_prefs_manager_set_side_pane_visible (visible);
+
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
+	                                      "ViewSidePane");
+
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
+
+	/* focus the document */
+	if (!visible && window->priv->active_tab != NULL)
+		gtk_widget_grab_focus (GTK_WIDGET (
+				gedit_tab_get_view (GEDIT_TAB (window->priv->active_tab))));
+}
+
 static void
 side_panel_size_allocate (GtkWidget     *widget,
 			  GtkAllocation *allocation,
@@ -2411,18 +2529,9 @@ side_panel_size_allocate (GtkWidget     *widget,
 	window->priv->side_panel_size = allocation->width;
 }
 
-static void
-side_panel_hide (GtkWidget   *panel,
-		 GeditWindow *window)
-{
-	_gedit_window_set_side_panel_visible (window, FALSE);
-}
-		 
 static void
 create_side_panel (GeditWindow *window)
 {
-	GtkAction *action;
-	gboolean visible;
 	GtkWidget *documents_panel;
 
 	gedit_debug (DEBUG_WINDOW);
@@ -2436,14 +2545,18 @@ create_side_panel (GeditWindow *window)
 	gtk_widget_set_size_request (window->priv->side_panel, 100, -1);  			 
 
 	g_signal_connect (window->priv->side_panel,
-			  "size_allocate",
+			  "size-allocate",
 			  G_CALLBACK (side_panel_size_allocate),
 			  window);
 
-	g_signal_connect (window->priv->side_panel,
-			  "hide",
-			  G_CALLBACK (side_panel_hide),
-			  window);
+	g_signal_connect_after (window->priv->side_panel,
+				"show",
+				G_CALLBACK (side_panel_visibility_changed),
+				window);
+	g_signal_connect_after (window->priv->side_panel,
+				"hide",
+				G_CALLBACK (side_panel_visibility_changed),
+				window);
 
 	gtk_paned_set_position (GTK_PANED (window->priv->hpaned),
 				MAX (100, gedit_prefs_manager_get_side_panel_size ()));
@@ -2453,17 +2566,30 @@ create_side_panel (GeditWindow *window)
 					      documents_panel,
 					      "Documents",
 					      GTK_STOCK_FILE);
+}
 
-	visible = gedit_prefs_manager_get_side_pane_visible ();
+static void
+bottom_panel_visibility_changed (GeditPanel  *bottom_panel,
+				 GeditWindow *window)
+{
+	gboolean visible;
+	GtkAction *action;
 
-	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					      "ViewSidePane");		
+	visible = GTK_WIDGET_VISIBLE (bottom_panel);
+
+	if (gedit_prefs_manager_bottom_panel_visible_can_set ())
+		gedit_prefs_manager_set_bottom_panel_visible (visible);
+
+	action = gtk_action_group_get_action (window->priv->action_group,
+					      "ViewBottomPane");
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
-	if (visible)
-		gtk_widget_show (window->priv->side_panel);
+	/* focus the document */
+	if (!visible && window->priv->active_tab != NULL)
+		gtk_widget_grab_focus (GTK_WIDGET (
+				gedit_tab_get_view (GEDIT_TAB (window->priv->active_tab))));
 }
 
 static void
@@ -2475,55 +2601,112 @@ bottom_panel_size_allocate (GtkWidget     *widget,
 }
 
 static void
-bottom_panel_hide (GtkWidget   *panel,
-		   GeditWindow *window)
+bottom_panel_item_removed (GeditPanel  *panel,
+			   GtkWidget   *item,
+			   GeditWindow *window)
 {
-	_gedit_window_set_bottom_panel_visible (window, FALSE);
+	if (gedit_panel_get_n_items (panel) == 0)
+	{
+		GtkAction *action;
+
+		gtk_widget_hide (GTK_WIDGET (panel));
+
+		action = gtk_action_group_get_action (window->priv->action_group,
+						      "ViewBottomPane");
+		gtk_action_set_sensitive (action, FALSE);
+	}
+}
+
+static void
+bottom_panel_item_added (GeditPanel  *panel,
+			 GtkWidget   *item,
+			 GeditWindow *window)
+{
+	/* if it's the first item added, set the menu item
+	 * sensitive and if needed show the panel */
+	if (gedit_panel_get_n_items (panel) == 1)
+	{
+		GtkAction *action;
+		gboolean show;
+
+		action = gtk_action_group_get_action (window->priv->action_group,
+						      "ViewBottomPane");
+		gtk_action_set_sensitive (action, TRUE);
+
+		show = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+		if (show)
+			gtk_widget_show (GTK_WIDGET (panel));
+	}
 }
 
 static void
 create_bottom_panel (GeditWindow *window) 
 {
-	GtkAction *action;
-	gboolean visible;
-
 	gedit_debug (DEBUG_WINDOW);
 
 	window->priv->bottom_panel = gedit_panel_new (GTK_ORIENTATION_HORIZONTAL);
-	
+
 	gtk_paned_pack2 (GTK_PANED (window->priv->vpaned), 
 			 window->priv->bottom_panel, 
-			 FALSE, 
+			 FALSE,
 			 FALSE);
 
 	g_signal_connect (window->priv->bottom_panel,
-			  "size_allocate",
+			  "size-allocate",
 			  G_CALLBACK (bottom_panel_size_allocate),
 			  window);
 
-  	g_signal_connect (window->priv->bottom_panel,
-			  "hide",
-			  G_CALLBACK (bottom_panel_hide),
-			  window);
+	g_signal_connect_after (window->priv->bottom_panel,
+				"show",
+				G_CALLBACK (bottom_panel_visibility_changed),
+				window);
+	g_signal_connect_after (window->priv->bottom_panel,
+				"hide",
+				G_CALLBACK (bottom_panel_visibility_changed),
+				window);
 
 	gtk_paned_set_position (GTK_PANED (window->priv->vpaned),
-				gedit_prefs_manager_get_bottom_panel_size ()); // CHECK set a min height as we do for the sidepane?
+				MAX (50, gedit_prefs_manager_get_bottom_panel_size ()));
+}
 
-	visible = gedit_prefs_manager_get_bottom_panel_visible();
+static void
+init_panels_visibility (GeditWindow *window)
+{
+	gedit_debug (DEBUG_WINDOW);
 
-	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					      "ViewBottomPanel");
+	/* side pane */
+	if (gedit_prefs_manager_get_side_pane_visible ())
+		gtk_widget_show (window->priv->side_panel);
 
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
+	/* bottom pane, it can be empty */
+	if (gedit_panel_get_n_items (GEDIT_PANEL (window->priv->bottom_panel)) > 0)
+	{
+		if (gedit_prefs_manager_get_bottom_panel_visible ())
+  			gtk_widget_show (window->priv->bottom_panel);
+	}
+	else
+	{
+		GtkAction *action;
+		action = gtk_action_group_get_action (window->priv->action_group,
+						      "ViewBottomPane");
+		gtk_action_set_sensitive (action, FALSE);
+	}
 
-	if(visible) 
-  		gtk_widget_show_all (window->priv->bottom_panel);
+	/* start track sensitivity after the initial state is set */
+  	g_signal_connect (window->priv->bottom_panel,
+			  "item_removed",
+			  G_CALLBACK (bottom_panel_item_removed),
+			  window);
+  	g_signal_connect (window->priv->bottom_panel,
+			  "item_added",
+			  G_CALLBACK (bottom_panel_item_added),
+			  window);
 }
 
 static void
 gedit_window_init (GeditWindow *window)
 {
+	static gboolean is_first = TRUE;
 	GtkWidget *main_box;
 
 	gedit_debug (DEBUG_WINDOW);
@@ -2542,11 +2725,9 @@ gedit_window_init (GeditWindow *window)
 	gtk_widget_show (main_box);
 
 	/* Add menu bar and toolbar bar */
-	gedit_debug_message (DEBUG_WINDOW, "Add menu bar and toolbar bar");
 	create_menu_bar_and_toolbar (window, main_box);
 
 	/* Add status bar */
-	gedit_debug_message (DEBUG_WINDOW, "Add statusbar");	
 	create_statusbar (window, main_box);
 
 	/* Add the main area */
@@ -2557,17 +2738,12 @@ gedit_window_init (GeditWindow *window)
   			    TRUE, 
   			    TRUE, 
   			    0);
-	gtk_widget_show (window->priv->hpaned);
 
-	gedit_debug_message (DEBUG_WINDOW, "Create side panel");		
-  	create_side_panel (window);
-  	  	
 	window->priv->vpaned = gtk_vpaned_new ();
   	gtk_paned_pack2 (GTK_PANED (window->priv->hpaned), 
   			 window->priv->vpaned, 
   			 TRUE, 
   			 FALSE);
-  	gtk_widget_show (window->priv->vpaned);
   	
 	gedit_debug_message (DEBUG_WINDOW, "Create gedit notebook");
 	window->priv->notebook = gedit_notebook_new ();
@@ -2577,13 +2753,32 @@ gedit_window_init (GeditWindow *window)
   			 TRUE);
   	gtk_widget_show (window->priv->notebook);  			 
 
-	gedit_debug_message (DEBUG_WINDOW, "Create bottom panel");
+	/* side and bottom panels */
+  	create_side_panel (window);
 	create_bottom_panel (window);
-	
-	/* Set visibility of panels */
-	// TODO
 
-	gedit_debug_message (DEBUG_WINDOW, "Connect signals");
+	/* restore paned positions as soon as we know the panes allocation.
+	 * This needs to be done only for the first window, the successive
+	 * windows are created by cloning the first one */
+	if (is_first)
+	{
+		is_first = FALSE;
+
+		window->priv->side_panel_size = gedit_prefs_manager_get_side_panel_size ();
+		window->priv->bottom_panel_size = gedit_prefs_manager_get_bottom_panel_size ();
+		g_signal_connect_after (window->priv->hpaned,
+					"map",
+					G_CALLBACK (hpaned_restore_position),
+					window);
+		g_signal_connect_after (window->priv->vpaned,
+					"map",
+					G_CALLBACK (vpaned_restore_position),
+					window);
+	}
+
+	gtk_widget_show (window->priv->hpaned);
+	gtk_widget_show (window->priv->vpaned);
+
 	/* Drag and drop support */
 	gtk_drag_dest_set (GTK_WIDGET (window),
 			   GTK_DEST_DEFAULT_MOTION |
@@ -2627,31 +2822,35 @@ gedit_window_init (GeditWindow *window)
 			  G_CALLBACK (notebook_popup_menu),
 			  window);
 
-	/* connect instead pf override, so that we can
+	/* connect instead of override, so that we can
 	 * share the cb code with the view */
 	g_signal_connect (window,
 			  "drag_data_received",
 	                  G_CALLBACK (drag_data_received_cb), 
 	                  NULL);
-	                  
-	gedit_debug_message (DEBUG_WINDOW, "Update plugins ui");	                  
-        gedit_plugins_engine_update_plugins_ui (window, TRUE);
-        
-       	gedit_debug_message (DEBUG_WINDOW, "END");	                  
+
+	gedit_debug_message (DEBUG_WINDOW, "Update plugins ui");
+	gedit_plugins_engine_update_plugins_ui (window, TRUE);
+
+	/* set visibility of panes.
+	 * This needs to be done after plugins activatation */
+	init_panels_visibility (window);
+
+	gedit_debug_message (DEBUG_WINDOW, "END");
 }
 
 GeditView *
 gedit_window_get_active_view (GeditWindow *window)
 {
 	GeditView *view;
-	
+
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), NULL);
-	
+
 	if (window->priv->active_tab == NULL)
 		return NULL;
-		
+
 	view = gedit_tab_get_view (GEDIT_TAB (window->priv->active_tab));
-	
+
 	return view;
 }
 
@@ -2659,13 +2858,13 @@ GeditDocument *
 gedit_window_get_active_document (GeditWindow *window)
 {
 	GeditView *view;
-	
+
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), NULL);
-	
+
 	view = gedit_window_get_active_view (window);
 	if (view == NULL)
 		return NULL;
-	
+
 	return GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 }
 
@@ -2692,7 +2891,7 @@ gedit_window_create_tab (GeditWindow *window,
 				tab,
 				-1,
 				jump_to);
-				
+
 	return tab;
 }
 
@@ -2708,14 +2907,14 @@ gedit_window_create_tab_from_uri (GeditWindow         *window,
 
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), NULL);
 	g_return_val_if_fail (uri != NULL, NULL);
-	
+
 	tab = _gedit_tab_new_from_uri (uri,
 				       encoding,
 				       line_pos,
 				       create);	
 	if (tab == NULL)
 		return NULL;
-		
+
 	gtk_widget_show (tab);	
 	
 	gedit_notebook_add_tab (GEDIT_NOTEBOOK (window->priv->notebook),
@@ -2826,195 +3025,23 @@ gedit_window_close_tabs (GeditWindow *window,
 
 	if (tabs == NULL)
 		return;
-		
+
 	window->priv->removing_tabs = TRUE;
 
 	while (tabs != NULL)
 	{
 		if (tabs->next == NULL)
 			window->priv->removing_tabs = FALSE;
-			
+
 		gedit_notebook_remove_tab (GEDIT_NOTEBOOK (window->priv->notebook),
 				   	   GEDIT_TAB (tabs->data));
 
 		tabs = g_list_next (tabs);
 	}
-	
+
 	g_return_if_fail (window->priv->removing_tabs == FALSE);
 }
 
-void
-_gedit_window_set_statusbar_visible (GeditWindow *window,
-				     gboolean     visible)
-{
-	GtkAction *action;
-	static gboolean recursione_guard = FALSE;
-
-	gedit_debug (DEBUG_WINDOW);
-
-	g_return_if_fail (GEDIT_IS_WINDOW (window));
-
-	if (recursione_guard)
-		return;
-
-	recursione_guard = TRUE;
-
-	visible = (visible != FALSE);
-		
-	if (visible)
-		gtk_widget_show (window->priv->statusbar);
-	else
-		gtk_widget_hide (window->priv->statusbar);
-
-	if (gedit_prefs_manager_statusbar_visible_can_set ())
-		gedit_prefs_manager_set_statusbar_visible (visible);
-		
-	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					      "ViewStatusbar");		
-		
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
-
-	recursione_guard = FALSE;		
-}
-
-void
-_gedit_window_set_toolbar_visible (GeditWindow *window,
-				   gboolean     visible)
-{
-	GtkAction *action;
-	static gboolean recursione_guard = FALSE;
-
-	gedit_debug (DEBUG_WINDOW);
-
-	g_return_if_fail (GEDIT_IS_WINDOW (window));
-
-	if (recursione_guard)
-		return;
-
-	recursione_guard = TRUE;
-	
-	visible = (visible != FALSE);
-
-	if (visible)
-		gtk_widget_show (window->priv->toolbar);
-	else
-		gtk_widget_hide (window->priv->toolbar);
-
-	if (gedit_prefs_manager_toolbar_visible_can_set ())
-		gedit_prefs_manager_set_toolbar_visible (visible);
-
-	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					      "ViewToolbar");		
-
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
-
-	recursione_guard = FALSE;
-}
-
-void
-_gedit_window_set_side_panel_visible (GeditWindow *window,
-				      gboolean     visible)
-{
-	GtkAction *action;
-	static gboolean recursion_guard = FALSE;
-	gboolean show = FALSE;
-
-	gedit_debug (DEBUG_WINDOW);
-
-	g_return_if_fail (GEDIT_IS_WINDOW (window));
-
-	if (recursion_guard)
-		return;
-
-	recursion_guard = TRUE;
-
-	visible = (visible != FALSE);
-
-	if (visible &&
-	    (GTK_WIDGET_VISIBLE (window->priv->side_panel) != visible))
-	{
-		gtk_widget_show (window->priv->side_panel);
-		show = TRUE;
-	}
-	else
-	{
-		gtk_widget_hide (window->priv->side_panel);
-		if (window->priv->active_tab)
-			gtk_widget_grab_focus (GTK_WIDGET (
-					gedit_tab_get_view (GEDIT_TAB (window->priv->active_tab))));
-	}
-
-	if (gedit_prefs_manager_side_pane_visible_can_set ())
-		gedit_prefs_manager_set_side_pane_visible (visible);
-
-	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					      "ViewSidePane");		
-
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
-
-	if (show)
-	{
-		/* g_print ("GRAB side panel\n"); */
-		gtk_widget_grab_focus (window->priv->side_panel);
-	}
-
-	recursion_guard = FALSE;
-}
-
-void
-_gedit_window_set_bottom_panel_visible (GeditWindow *window,
-					gboolean     visible)
-{
-	GtkAction *action;
-	static gboolean recursion_guard = FALSE;
-	gboolean show = FALSE;
-
-	gedit_debug (DEBUG_WINDOW);
-
-	g_return_if_fail (GEDIT_IS_WINDOW (window));
-
-	if (recursion_guard)
-		return;
-
-	recursion_guard = TRUE;
-
-	visible = (visible != FALSE);
-
-	if (visible &&
-	    (GTK_WIDGET_VISIBLE (window->priv->bottom_panel) != visible))
-	{
-		gtk_widget_show (window->priv->bottom_panel);
-		show = TRUE;
-	}
-	else
-	{
-		gtk_widget_hide (window->priv->bottom_panel);
-		if (window->priv->active_tab)
-			gtk_widget_grab_focus (GTK_WIDGET (
-					gedit_tab_get_view (GEDIT_TAB (window->priv->active_tab))));
-	}
-
-	if (gedit_prefs_manager_bottom_panel_visible_can_set ())
-		gedit_prefs_manager_set_bottom_panel_visible (visible);
-
-	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					      "ViewBottomPanel");		
-
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
-
-	if (show)
-	{
-		/* g_print ("GRAB bottom panel\n"); */
-		gtk_widget_grab_focus (window->priv->bottom_panel);
-	}
-
-	recursion_guard = FALSE;
-}
-
 GeditWindow *
 _gedit_window_move_tab_to_new_window (GeditWindow *window,
 				      GeditTab    *tab)
@@ -3151,21 +3178,23 @@ gedit_window_get_unsaved_documents (GeditWindow *window)
 
 void 
 _gedit_window_set_saving_session_state (GeditWindow *window,
-				        gboolean     saving_session)
+					gboolean     saving_session)
 {
+	GeditWindowState old_state;
+
 	g_return_if_fail (GEDIT_IS_WINDOW (window));
-		
-	GeditWindowState old_state = window->priv->state;
 	
+	old_state = window->priv->state;
+
 	if (saving_session)
 		window->priv->state |= GEDIT_WINDOW_STATE_SAVING_SESSION;
 	else
 		window->priv->state &= ~GEDIT_WINDOW_STATE_SAVING_SESSION;
-		
+
 	if (old_state != window->priv->state)
 	{
 		set_sensitivity_according_to_window_state (window);
-		
+
 		g_object_notify (G_OBJECT (window), "state");
 	}
 }
